Package: tinytest
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: GPL-3
Title: Lightweight and Feature Complete Unit Testing Framework
Type: Package
LazyLoad: yes
Authors@R: c( person("Mark", "van der Loo"
              , role    = c("aut","cre")
              , email   = "mark.vanderloo@gmail.com"
              , comment = c(ORCID="0000-0002-9807-4686"))
            )
Description: Provides a lightweight (zero-dependency) and easy to use 
    unit testing framework. Main features: install tests with 
    the package. Test results are treated as data that can be stored and 
    manipulated. Test files are R scripts interspersed with test commands, that
    can be programmed over. Fully automated build-install-test sequence for 
    packages. Skip tests when not run locally (e.g. on CRAN). Flexible and 
    configurable output printing. Compare computed output with output stored 
    with the package. Run tests in parallel. Extensible by other packages.
    Report side effects.
Version: 1.4.3
URL: https://github.com/markvanderloo/tinytest
BugReports: https://github.com/markvanderloo/tinytest/issues
Depends: R (>= 3.0.0)
Imports: parallel, utils
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-03-21 14:09:31 UTC; mark
Author: Mark van der Loo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9807-4686>)
Repository: CRAN
Date/Publication: 2026-03-24 05:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 18:39:32 UTC; unix
