# Generated by roxygen2: do not edit by hand

S3method("[",tinytests)
S3method(as.data.frame,tinytests)
S3method(format,tinytest)
S3method(print,tinytest)
S3method(print,tinytests)
S3method(summary,tinytests)
export(all_fail)
export(all_pass)
export(any_fail)
export(any_pass)
export(at_home)
export(build_install_test)
export(exit_file)
export(exit_if_not)
export(expect_equal)
export(expect_equal_to_reference)
export(expect_equivalent)
export(expect_equivalent_to_reference)
export(expect_error)
export(expect_false)
export(expect_identical)
export(expect_inherits)
export(expect_length)
export(expect_match)
export(expect_message)
export(expect_null)
export(expect_silent)
export(expect_stdout)
export(expect_true)
export(expect_warning)
export(get_call_wd)
export(ignore)
export(puppy)
export(register_tinytest_extension)
export(report_side_effects)
export(run_test_dir)
export(run_test_file)
export(setup_tinytest)
export(test_all)
export(test_package)
export(tinytest)
export(using)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(utils,capture.output)
importFrom(utils,file_test)
importFrom(utils,getFromNamespace)
importFrom(utils,install.packages)
